package net.ihe.gazelle.proxy.gui;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.ejb.Remove;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.persistence.EntityManager;

import net.ihe.gazelle.common.filter.HibernateDataModel;
import net.ihe.gazelle.proxy.dao.MessageFilterStep;
import net.ihe.gazelle.proxy.dao.ProxyDAO;
import net.ihe.gazelle.proxy.dao.ProxyDAOTM;
import net.ihe.gazelle.proxy.model.message.AbstractMessage;
import net.ihe.gazelle.proxy.model.message.DicomMessage;
import net.ihe.gazelle.proxy.model.message.HL7Message;
import net.ihe.gazelle.proxy.model.message.HTTPMessage;
import net.ihe.gazelle.proxy.model.message.RawMessage;
import net.ihe.gazelle.proxy.model.message.SyslogMessage;
import net.ihe.gazelle.proxy.model.tm.Step;
import net.ihe.gazelle.proxy.netty.ChannelType;
import net.ihe.gazelle.proxy.netty.channel.ProxySide;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.faces.Redirect;

@Name("messagesStepBean")
@Scope(ScopeType.PAGE)
public class MessagesStepBean {

	private transient HibernateMessageDataModel<AbstractMessage> messageDataModel;
	private MessageFilterStep messageFilterStep;

	@In
	private EntityManager entityManager;

	private Step step = null;

	public void clearFilter() {
		rebuildDataModel();
	}

	@Create
	public void createModel() {
		messageFilterStep = new MessageFilterStep(step);
		rebuildDataModel();
	}

	@Remove
	@Destroy
	public void destroy() {
	}

	public HibernateDataModel<?> getMessageDataModel() {
		return messageDataModel;
	}

	public List<ChannelType> getMessageTypes() {
		return Arrays.asList(ChannelType.values());
	}

	public List<ProxySide> getProxySides() {
		return Arrays.asList(ProxySide.values());
	}

	public SelectItem[] getDicomAffectedSopClassUIDs() {
		Map<String, String> mapOfDicomUID = DicomMessage.getMapOfDicomUID(entityManager);
		List<String> result = new ArrayList<String>(mapOfDicomUID.values());
		Collections.sort(result);
		return getSelectItems(result);
	}

	private SelectItem[] getSelectItems(List<String> list) {
		SelectItem[] result = new SelectItem[list.size() + 1];
		int i = 0;
		result[i] = new SelectItem(null, "--");
		i++;
		for (String option : list) {
			result[i] = new SelectItem(option, option);
			i++;
		}
		return result;
	}

	public SelectItem[] getDicomRequestedSopClassUIDs() {
		return getDicomAffectedSopClassUIDs();
	}

	public SelectItem[] getDicomCommandFields() {
		Map<Integer, String> mapOfCommandFields = DicomMessage.getMapOfCommandFields(entityManager);
		List<String> result = new ArrayList<String>(mapOfCommandFields.values());
		Collections.sort(result);
		return getSelectItems(result);
	}

	public boolean isFilterDates() {
		return messageFilterStep.isFilterDates();
	}

	public boolean isFilterPathFrom() {
		return messageFilterStep.isFilterPathFrom();
	}

	public String getStepFilter() {
		return messageFilterStep.getStepFilter();
	}

	public String linkedLabel(AbstractMessage message) {
		if (message instanceof HTTPMessage) {
			HTTPMessage httpMessage = (HTTPMessage) message;
			if (httpMessage.getMatchingMessage() == null) {
				return "";
			}
			if (httpMessage.getMatchingMessage().getId() == null) {
				return "";
			}
			return Integer.toString(httpMessage.getMatchingMessage().getId());
		}
		return "";
	}

	public Boolean linkedRendered(AbstractMessage message) {
		if (message instanceof HTTPMessage) {
			HTTPMessage httpMessage = (HTTPMessage) message;
			if (httpMessage.getMatchingMessage() != null) {
				return Boolean.TRUE;
			}
		}
		return Boolean.FALSE;
	}

	private void rebuildDataModel() {
		messageDataModel = new HibernateMessageDataModel<AbstractMessage>(messageFilterStep);
	}

	public String redirectToLinked(AbstractMessage message) {
		if (message instanceof HTTPMessage) {
			HTTPMessage httpMessage = (HTTPMessage) message;
			if (httpMessage.getMatchingMessage() == null) {
				return null;
			}
			if (httpMessage.getMatchingMessage().getId() == null) {
				return null;
			}
			return "/messages/http.seam?id=" + httpMessage.getMatchingMessage().getId();
		}
		return null;
	}

	public void redirectToMessageFromURL() {
		FacesContext fc = FacesContext.getCurrentInstance();
		String id = (String) fc.getExternalContext().getRequestParameterMap().get("id");
		AbstractMessage message = ProxyDAO.getMessageByID(Integer.parseInt(id), entityManager);

		Redirect redirect = Redirect.instance();
		redirect.setParameter("id", id);

		String viewId = "";
		if (message instanceof HL7Message) {
			viewId = "/messages/hl7.xhtml";
		}
		if (message instanceof RawMessage) {
			viewId = "/messages/raw.xhtml";
		}
		if (message instanceof DicomMessage) {
			viewId = "/messages/dicom.xhtml";
		}
		if (message instanceof SyslogMessage) {
			viewId = "/messages/syslog.xhtml";
		}
		if (message instanceof HTTPMessage) {
			viewId = "/messages/http.xhtml";
		}

		redirect.setViewId(viewId);
		redirect.execute();
	}

	public String redirectToMessage(AbstractMessage message) {
		if (message.getId() == null) {
			return null;
		}
		if (message instanceof HL7Message) {
			return "/messages/hl7.seam?id=" + message.getId();
		}
		if (message instanceof RawMessage) {
			return "/messages/raw.seam?id=" + message.getId();
		}
		if (message instanceof DicomMessage) {
			return "/messages/dicom.seam?id=" + message.getId();
		}
		if (message instanceof SyslogMessage) {
			return "/messages/syslog.seam?id=" + message.getId();
		}
		if (message instanceof HTTPMessage) {
			return "/messages/http.seam?id=" + message.getId();
		}
		return null;
	}

	public void setFilterDates(boolean filterDates) {
		messageFilterStep.setFilterDates(filterDates);
	}

	public void setFilterPathFrom(boolean filterPathFrom) {
		messageFilterStep.setFilterPathFrom(filterPathFrom);
	}

	public void init() {
		Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
		String testStepId = params.get("id");
		List<Step> steps = ProxyDAOTM.getStepsByTMId(Integer.parseInt(testStepId), entityManager);
		if (steps.size() > 0) {
			step = steps.get(0);
		}
		messageFilterStep = new MessageFilterStep(step);

		rebuildDataModel();
	}

	public boolean isFilterPathReceiver() {
		return messageFilterStep.isFilterPathReceiver();
	}

	public void setFilterPathReceiver(boolean filterPathReceiver) {
		messageFilterStep.setFilterPathReceiver(filterPathReceiver);
	}

}
