package net.ihe.gazelle.proxy.listeners;

import net.ihe.gazelle.proxy.model.message.RawMessage;
import net.ihe.gazelle.proxy.model.message.SyslogMessage;
import net.ihe.gazelle.proxy.netty.channel.ProxySide;
import net.ihe.gazelle.proxy.netty.protocols.syslog.SyslogData;

public class SyslogEventListener extends GazelleProxyEventListener<SyslogData, byte[]> {

	public SyslogEventListener(MessageListener messageListener) {
		super(messageListener);
	}

	@Override
	public void onRequest(SyslogData request, String requesterIp, int requesterPort, int proxyProviderPort,
			String responderIp, int responderPort, int providerChannelId, int consumerChannelId) {
		synchronized (this) {
			SyslogMessage syslogMessage = new SyslogMessage(requesterIp, requesterPort, proxyProviderPort, responderIp,
					responderPort, providerChannelId, consumerChannelId, request);
			messageListener.processMessage(syslogMessage);
		}
	}

	@Override
	public void onResponse(byte[] response, String requesterIp, int requesterPort, int proxyProviderPort,
			String responderIp, int responderPort, int providerChannelId, int consumerChannelId) {
		// Messages are coming from different threads!
		synchronized (this) {
			RawMessage rawmessage = new RawMessage(requesterIp, requesterPort, proxyProviderPort, responderIp,
					responderPort, providerChannelId, consumerChannelId, ProxySide.RESPONSE);
			rawmessage.setMessageReceived(response);
			messageListener.processMessage(rawmessage);
		}
	}
}
