package net.ihe.gazelle.proxy.netty;

import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import net.ihe.gazelle.proxy.netty.channel.PipelineFactoryProvider;

import org.jboss.netty.bootstrap.ServerBootstrap;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelHandler;
import org.jboss.netty.channel.socket.ClientSocketChannelFactory;
import org.jboss.netty.channel.socket.ServerSocketChannelFactory;
import org.jboss.netty.channel.socket.nio.NioClientSocketChannelFactory;
import org.jboss.netty.channel.socket.nio.NioServerSocketChannelFactory;

public abstract class Proxy<REQU, RESP> {

	private static final ExecutorService executor = Executors.newCachedThreadPool();

	// Set up the client socket factory (for connecting to remote host).
	public static final ClientSocketChannelFactory clientChannelFactory = new NioClientSocketChannelFactory(executor,
			executor);
	public static final ServerSocketChannelFactory serverChannelFactory = new NioServerSocketChannelFactory(executor,
			executor);

	private transient Channel serverChannel;
	private ConnectionConfig connectionConfig;
	private transient ProxyEventListener<REQU, RESP> proxyEventListener;

	public Proxy(ProxyEventListener<REQU, RESP> proxyEventListener, ConnectionConfig connectionConfig) {
		super();

		this.serverChannel = null;
		this.connectionConfig = connectionConfig;
		this.proxyEventListener = proxyEventListener;
	}

	public abstract List<ChannelHandler> getDecodersForResponse();

	public abstract List<ChannelHandler> getDecodersForRequest();

	public abstract REQU handleRequest(Object message);

	public abstract RESP handleResponse(Object message);

	public int getProxyProviderPort() {
		return connectionConfig.getProxyProviderPort();
	}

	public String getProxyConsumerHost() {
		return connectionConfig.getProxyConsumerHost();
	}

	public int getProxyConsumerPort() {
		return connectionConfig.getProxyConsumerPort();
	}

	public ChannelType getChannelType() {
		return connectionConfig.getChannelType();
	}

	public List<ChannelHandler> getHandlersForProxyProvider() {
		if (connectionConfig != null) {
			return connectionConfig.getHandlersForProxyProvider();
		} else {
			return new ArrayList<ChannelHandler>(0);
		}
	}

	public List<ChannelHandler> getHandlersForProxyConsumer() {
		if (connectionConfig != null) {
			return connectionConfig.getHandlersForProxyConsumer();
		} else {
			return new ArrayList<ChannelHandler>(0);
		}
	}

	public ProxyEventListener<REQU, RESP> getProxyEventListener() {
		return proxyEventListener;
	}

	public boolean isOpen() {
		if (serverChannel != null && serverChannel.isOpen()) {
			return true;
		}
		return false;
	}

	public void start() {
		if (serverChannel != null) {
			throw new IllegalStateException("Already started");
		}
		// Configure the TCP/IP server.
		ServerBootstrap bootstrap = new ServerBootstrap(serverChannelFactory);
		// Set up the event pipeline factory.
		bootstrap.setPipelineFactory(new PipelineFactoryProvider<REQU, RESP>(this));
		// Bind and start to accept incoming connections.
		serverChannel = bootstrap.bind(new InetSocketAddress(getProxyProviderPort()));
	}

	public void stop() {
		if (serverChannel == null) {
			throw new IllegalStateException("Not started");
		}
		serverChannel.close().awaitUninterruptibly();
		serverChannel = null;
	}

}
